#include "commands.h"
#include "widgetfactory.h"
#include <QMetaProperty>
#include "props_functions.h"
DeleteCommand::DeleteCommand ( WidgetManager & W , QUndoCommand *parent)
    : QUndoCommand (parent)
{
    WM = & W ;
    selList = WM->getCurrentSelList () ;

}

void DeleteCommand::undo ()
{
    WM->setDeleted ( false , selList ) ;
}

void DeleteCommand::redo ()
{
    WM->setDeleted ( true , selList ) ;
}

MorphCommand::MorphCommand ( WidgetManager &W , QString className , QUndoCommand *parent)
    : QUndoCommand ( parent ) , WM ( & W ) , newClassName ( className )
{
    selList = WM->getCurrentSelList () ;
    foreach ( QWidget * w , selList )
        if ( w )
            prevClassNames.append ( w->metaObject()->className() ) ;
    widgets = WM->morphWidgetsInto ( selList , newClassName ) ;
}

void MorphCommand::undo ()
{
    WM->setDeleted( true , widgets ) ;
    WM->setDeleted( false , selList ) ;
}

void MorphCommand::redo ()
{
    WM->setDeleted( false , widgets ) ;
    WM->setDeleted( true , selList ) ;
}

MoveAndDelCommand::MoveAndDelCommand ( WidgetManager & W , QWidget *par ,
                                       QPoint hs, QUndoCommand *parent ) : QUndoCommand ( parent ) , parentWidget ( par ) , hotSpot ( hs )
{
    WM = & W ;
    QPoint hotSpot = parentWidget->mapFromGlobal ( QCursor::pos () ) ;
    offset = hotSpot - hs ;
    widgets = WM->widgetsFromClbd ( parentWidget , offset ) ;
    oldWidgets = WM->getDragWidgets () ;
}

MoveAndDelCommand::~MoveAndDelCommand ()
{
    foreach ( QPointer< QWidget > s , widgets )
        if ( s )
            s->deleteLater () ;
    foreach ( QPointer< QWidget > s , oldWidgets )
        if ( s )
            s->deleteLater () ;
}

void MoveAndDelCommand::undo ()
{
    WM->setDeleted ( false , oldWidgets ) ;
    WM->setDeleted ( true , widgets ) ;
}

void MoveAndDelCommand::redo ()
{
    WM->setDeleted ( true , oldWidgets ) ;
    WM->setDeleted ( false , widgets ) ;
}

MoveCommand::MoveCommand (  WidgetManager & W , QPoint dlt ,
                          QUndoCommand *parent ) : QUndoCommand ( parent )  , movingId ( 0 )
{
    WM = & W ;
    delta = dlt ;
    selList = WM->getCurrentSelList () ;
    movingId = movingIdst ;
}

quint64 MoveCommand::movingIdst = 0 ;

void MoveCommand::undo ()
{
    WM->moveWidgets ( - delta , selList ) ;
}

void MoveCommand::redo ()
{
    WM->moveWidgets ( delta , selList ) ;
}

bool MoveCommand::mergeWith ( const QUndoCommand * command )
{
    const MoveCommand *moveCommand = static_cast< const MoveCommand * > ( command ) ;
    if ( moveCommand->selList != selList || movingId != moveCommand->movingId )
        return false ;
    delta += moveCommand->delta ;
    return true ;
}

int MoveCommand::id  () const
{
    return 0 ;
}

ResizeCommand::ResizeCommand ( WidgetManager &W , QPoint dlt , int i ,
                              QUndoCommand *parent ) : QUndoCommand ( parent ) ,
    resizingId ( 0 ),  direction ( i )
{
    WM = & W ;
    delta = dlt ;
    WM->deleteSelDublicates () ;
    selList = WM->getCurrentSelList () ;
    resizingId = resizingIdst ;
}

quint64 ResizeCommand::resizingIdst = 0 ;

void ResizeCommand::undo ()
{
    WM->resizeWidgets ( - delta , direction , selList ) ;
}

void ResizeCommand::redo ()
{ 
    WM->resizeWidgets ( delta , direction , selList ) ;
}


bool ResizeCommand::mergeWith ( const QUndoCommand *command )
{
    const ResizeCommand * resizeCommand = static_cast< const ResizeCommand * > ( command ) ;
    if ( resizeCommand->selList != selList || resizingId != resizeCommand->resizingId )
        return false ;
    delta += resizeCommand->delta ;
    return true ;
}

int ResizeCommand::id  () const
{
    return 1 ;
}

EditCommand::EditCommand ( WidgetManager &W , QVariantMap pr , QUndoCommand *parent )
    : QUndoCommand ( parent )
{
    WM = & W ;

    props = pr ;
    selList = WM->getCurrentSelList () ;
    foreach ( QPointer< QWidget > widget , selList )
    {
        if ( widget )
        {
            QVariantMap pr ;

            QMap < QString , QVariant >::const_iterator i = props.constBegin () ;
            while  (i != props.constEnd () )
            {
                QVariant v = PF::readProperty( widget , i.key() ) ;
                pr.insert (  i.key () , v ) ;

                ++ i ;
            }

            oldProps.append ( pr ) ;
        }
    }
}

void EditCommand::undo ()
{
    for ( int j = 0 ; j < selList.count () ; j ++ )
        WM->applyPropertiesToWidgets ( oldProps [ j ] , QList < QPointer< QWidget > > () << selList [ j ] ) ;
}

void EditCommand::redo ()
{
    WM->applyPropertiesToWidgets ( props , selList ) ;
}

AlignCommand::AlignCommand ( WidgetManager * W ,
                            WidgetManager::AligningSide s ,
                            QUndoCommand * parent ) :
    QUndoCommand ( parent ) , WM ( W ) , aSide ( s )
{
    selList = WM->getCurrentSelList () ;
    foreach ( QPointer< QWidget > widget , selList )
    {
        if ( widget )
            oldPos.insert ( widget , widget->pos () ) ;
    }
}

void AlignCommand::undo ()
{
    for ( int i = 0 ; i < oldPos.count () ; i ++ )
    foreach ( QPointer< QWidget > widget , selList )
    {
        if ( oldPos.keys () [ i ] )
            WM->setWidgetPos ( QList< QPointer < QWidget > >  () << oldPos.keys () [ i ] , oldPos.values () [ i ] ) ;
    }
}
void AlignCommand::redo ()
{
    WM->alignWidgets ( selList , aSide ) ;
}

